# hashpk - Routines for rapid install and lookup of tokens:
#	   hash, initlu, instal, lookup, uninst
#
# Copyright (c) 1988 by Michael J. Harrison
#
# Revised MJH, 21-Jun-86: Add uninst to uninstall (i.e. delete) name
# Revised MJH, 10-Jan-87: Switch to str... string functions
# Revised MJH,	6-Aug-87: Change uninst to make freed space available
# Revised MJH, 19-Aug-87: Fix bug in above change
# Revised MJH,  9-Sep-88: For INTEGER*1 chars
# M.J.Harrison 10-Apr-98: Linux

include dstd
include drmax

define FREED_INDEX (MAXTBL+1)	# -FREED_INDEX ^= any inactive index

#----------------------------------------------------------------------
# deftog - Toggle definition active or inactive (temporarily)
#----------------------------------------------------------------------

subroutine deftog(inamp)
integer inamp			# Index in namptr of definition

include clook

    namptr(inamp) = -namptr(inamp)
    return
    end		# of deftog

#----------------------------------------------------------------------
# hash - hash function for instal/lookup
#----------------------------------------------------------------------
#   uses standard division method

integer function hash(name)
character name(MAXTOK)

include clook	    # for HASHK only

    integer i, sum

    sum = 0
    for (i = 1; name(i) ^= EOS; i = i + 1)
	sum = sum + name(i)
    hash = mod(sum, HASHK)

    return
    end 	# of hash

#----------------------------------------------------------------------
# initlu - initialize token lookup table
#----------------------------------------------------------------------

subroutine initlu

include clook

    integer i

    free = MAXPTR - 1
    do i = 1, free, 2
	namptr(i) = 0
    lastt = 0

    return
    end 	# of initlu

#----------------------------------------------------------------------
# instal - add name and definition to table
#----------------------------------------------------------------------
#   new version to go with hash table (must now do lookup first).

logical function instal(name, defn)
character name(MAXTOK), defn(MAXDEF)

#  Returns: YES if installed, NO if already defined (not installed)

include clook
intfunc lookup, strcpy, strlen

    integer dlen, junk, nlen
    character cdummy(MAXDEF)

    instal = NO
    if (lookup(name, cdummy, NO) == 0) {# if not already in table ...
	instal = YES			# (lookup set pos as slot to add at)
	if (namptr(pos) > 0) {		# hashed posn occupied, find free slot
	    while (free > 0 & namptr(free) ^= 0 &
		   namptr(free) ^= -FREED_INDEX)
		free = free - 2
	    if (free <= 0)
		call error("Hash table full (instal).")
	    else {
		namptr(pos + 1) = free	# link to previous entry
		pos = free		# point at new slot
		}
	    }
	nlen = strlen(name) + 1
	dlen = strlen(defn) + 1
	if (lastt + nlen + dlen > MAXTBL)
	    call error("Name/defn table full (instal).")
	else {
	    if (namptr(pos) == 0)	# if not reclaimed deleted slot,
		namptr(pos + 1) = 0	# mark as end of chain
	    namptr(pos) = lastt + 1	# set index of table entry in slot
	    junk = strcpy(table(lastt+1), name)
	    junk = strcpy(table(lastt+nlen+1), defn)
	    lastt = lastt + nlen + dlen
	    }
	}

    return
    end 	# of instal

#----------------------------------------------------------------------
# lookup - locate name, return corresponding argument
#----------------------------------------------------------------------
#   Algorithm C from Knuth Vol. 3, p 514
#   This is chosen because it has a good performance for unsuccessful
#   searches even when the table is nearly full. In Ratfor, most
#   searches are unsuccessful (most tokens are not DEFINEd).
#
#   Revised MJH, 25-Dec-90: Return index of found entry in namptr

integer function lookup(name, defn, docopy)
character name(MAXTOK), defn(MAXDEF)
logical docopy			# YES iff copy name (if found) to defn

#  Returns: Index in namptr of entry if found, and (if docopy TRUE)
#		defined string is copied to defn; pos holds index of
#		predecessor of matched entry or -(index of matched
#		entry) if no predecessor
#	    0 if not found; pos is left pointing at last namptr entry
#		in chain (or unused slot), so instal can extend chain

include clook
intfunc hash, strcmp, strcpy, strlen

    integer i, junk, inamp
    integer lpos, prepos	# in the loop, lpos looks ahead one entry

    lpos = 2 * hash(name) + 1	# lpos points at 0,? or index,next
    prepos = -lpos		# "Predecessor" if actually none
    inamp = 0
    if (namptr(lpos) == 0)	# Empty slot, so not found
	pos = lpos		# Return its index
    else {
	while (inamp == 0 & lpos ^= 0) {
	    i = namptr(lpos)	# (i < 0 if deleted or inactive entry)
	    if (i > 0 & strcmp(table(i), name) == 0)	# try match
		inamp = lpos		# i points at match entry
	    else {
		prepos = lpos		# save current entry as predecessor
		lpos = namptr(lpos + 1) # set current to next (or end)
		}
	    }
	pos = prepos
	if (inamp ^= 0 & docopy) {  # if requested, copy match's define string
	    i = i + strlen(table(i)) + 1    # i is index of name in table
	    junk = strcpy(defn, table(i))
	    }
	}

    lookup = inamp
    return
    end 	# of lookup

#----------------------------------------------------------------------
# uninst - locate name, delete entry and free up its space
#----------------------------------------------------------------------

logical function uninst(name)
character name(MAXTOK)

#  Returns: TRUE if name found, else FALSE

#  Notes:   Freeing up an entry is expensive, but infrequent

include clook
intfunc lookup, strlen

    integer adj 		# amount to shift strings in table
    integer curpos		# index to namptr (of entry to delete)
    integer delent, ent, nxtent # indexes to table (of string to delete, next)
    character cdummy(MAXDEF)

    # Locate name.  If found, mark entry as free

    uninst = (lookup(name, cdummy, NO) ^= 0)
    if (uninst) {
	if (pos > 0)			# Point at entry to free
	    curpos = namptr(pos + 1)	#  from predecessor
	else
	    curpos = -pos		#  or from self if first in chain
	delent = namptr(curpos) 	# Save table index of str to delete
	if (namptr(curpos + 1) == 0) {	# If last on chain, really free
	    namptr(curpos) = 0
	    if (pos > 0)		#  if entry to free has predecessor
		namptr(pos + 1) = 0	#  make chain end at predecessor
	    }
	else				# Else, just mark freed
	    namptr(curpos) = -FREED_INDEX
	if (free < curpos)		# Ensure we'll pick it as freed
	    free = curpos

	# Overlay higher strings on deleted string pair.

	ent = delent
	adj = strlen(table(ent)) + 1	# Amount to shift down for name
	adj = adj + strlen(table(ent+adj)) + 1	# + defn of deleted pair
	nxtent = ent + adj
	while (nxtent <= lastt) {
	    table(ent) = table(nxtent)
	    ent = ent + 1
	    nxtent = nxtent + 1
	    }
	lastt = lastt - adj		# Adjust high point in table

	# Adjust pointers > deleted string (use curpos to index namptr
	# table, since we don't need value of curpos any more).

	for (curpos = MAXPTR - 1; curpos > 0; curpos = curpos - 2) {
	    if (namptr(curpos) > delent)
		namptr(curpos) = namptr(curpos) - adj
	    }
	}

    return
    end 	# of uninst
