#-----------------------------------------------------------------------
#
#  FPTHPK - Procedures for manipulating full pathnames:
#
#  FNEQL      - Compare two pathnames to see if filenames equal
#  FNOFFP     - Strip filename from pathname and copy it
#  FPOFFP     - Strip filepath from pathname and copy it
#  FPSPLT     - Return index of start of filename in pathname
#
#-----------------------------------------------------------------------
#
#  FNEQL      - Compare two pathnames to see if filenames equal
#  Synopsis:	val = FNEQL (path1, path2)
#		int	val	    YES if equal, else NO
#		char(*) path1	    Full MS-DOS pathname
#		char(*) path2	    Ditto
#
#  Purpose:	Allow filename comparisons with full pathnames
#  Notes:
#  Method:	Scan each pathname from left to find last "\" or ":".
#		Compare filename parts only.
#
#  Written MJH, 13-Oct-85:
#  Revised MJH, 14-Dec-85: To use fpsplt
#  Revised MJH, 11-Jan-87: Use new str... functions
#  M.J.Harrison  9-Apr-98: For Linux
#
#-------------- Copyright (C) 1985 by Michael J. Harrison --------------

include "dmach.i"
include dstd
include "dmetac.i"

#****** Procedure Entry ******#

logical function FNEQL(path1, path2)
character path1(ARB), path2(ARB)

#====== Externals ======#

intfunc fpsplt, strcmp

    #====== Local Declarations ======#

    integer i, j

    #====== Executable Code ======#

    i = fpsplt(path1)
    j = fpsplt(path2)
    fneql = (strcmp(path1(i), path2(j)) == 0)

    return
    end # of FNEQL

#-----------------------------------------------------------------------
#
#  FNOFFP     - Strip filename from pathname and copy it
#  Synopsis:	val = FNOFFP(pathname, filename)
#		int	val	    Length of filename (excl. EOS)
#		char(*) pathname    Full MS-DOS pathname
#		char(*) filename    MS-DOS filename (nnnnnnnn.ext)
#
#  Purpose:	Allow filename comparisons when one has path
#  Notes:
#  Method:	Scan pathname from left to find last "\" or ":".
#		Copy from (this+1) to EOS to filename.
#
#  Written MJH, 13-Oct-85:
#  Revised MJH, 14-Dec-85: To use fpsplt
#
#-------------- Copyright (C) 1985 by Michael J. Harrison --------------

#****** Procedure Entry ******#

integer function FNOFFP(path, fn)
character path(ARB), fn(ARB)

#====== Externals ======#

intfunc fpsplt

    #====== Local Declarations ======#

    integer i, j

    #====== Executable Code ======#

    j = 1
    for (i = fpsplt(path); path(i) ^= EOS; i = i + 1)
	{
	fn(j) = path(i)
	j = j + 1
	}
    fn(j) = EOS

    fnoffp = j - 1

    return
    end # of FNOFFP

#-----------------------------------------------------------------------
#
#  FPOFFP     - Strip filepath from pathname and copy it
#  Synopsis:	val = FPOFFP(pathname, filepath)
#		int	val	    Length of filepath (excl. EOS)
#		char(*) pathname    Full MS-DOS pathname
#		char(*) filepath    MS-DOS filepath ([d:][\][p\]...)
#
#  Purpose:	Allow filepath to be prepended to filename
#  Notes:
#  Method:	Locate start of filename.  Copy first char to (this-1)
#		to filepath.
#
#  Written MJH, 14-Dec-85:
#
#-------------- Copyright (C) 1985 by Michael J. Harrison --------------

#****** Procedure Entry ******#

integer function FPOFFP(path, fp)
character path(ARB), fp(ARB)

#====== Externals ======#

intfunc fpsplt

    #====== Local Declarations ======#

    integer i, j, k

    #====== Executable Code ======#

    j = 1
    k = fpsplt(path)		# k is index of start of filename
    for (i = 1; i < k; i = i + 1)
	{
	fp(j) = path(i)
	j = j + 1
	}
    fp(j) = EOS

    fpoffp = j - 1

    return
    end # of FPOFFP

#-----------------------------------------------------------------------
#
#  FPSPLT     - Return index of start of filename in pathname
#  Synopsis:	val = FPSPLT(path)
#		int	val	    Index of start of filename
#		char(*) path	    Full MS-DOS pathname
#
#  Purpose:	Allow filename and filepath manipulation
#  Notes:
#  Method:	Scan pathname from left to find last "\" or ":".
#		Filename starts at next char.
#
#  Written MJH, 14-Dec-85:
#  M.J.Harrison  3-Apr-98: Linux: character comparisons
#
#-------------- Copyright (C) 1985 by Michael J. Harrison --------------

#****** Procedure Entry ******#

integer function FPSPLT(path)
character path(ARB)

    #====== Local Declarations ======#

    integer i, j

    #====== Executable Code ======#

    i = 1			# value if no drive or path
    for (j = 1; path(j) ^= EOS; j = j + 1)
	{
	if (path(j) == PATHSEP | path(j) == COLON)
	    i = j + 1
	}

    fpsplt = i

    return
    end # of FPSPLT
