/* rpfuncs.c    Ratfor primitive functions, for Linux
 *
 * Mike Harrison 01/22/2005: Changes for PowerPC (Mac/OS X)
 * Mike Harrison 10/15/2006: Changes for GNU f95
 * Mike Harrison 04/16/2014: Changes for GNU gfortran, OS X and MinGW
 * Mike Harrison 12/28/2017: Change MinGW predefined macro to __MINGW32__
 *
 * Copyright (c) 1998-2017 Michael J. Harrison.  All rights reserved.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <utime.h>
#include <glob.h>

#ifdef ISF77
extern int      iargc_(int *dummy);
extern int      getarg_(int *k, char *s, int slen);
#endif

/* Probably vain attempt to support in Linux environment
 * the interface that made sense for DOS.
 */
typedef struct {                /* The DOS DTA */
        char    fi0[21];
        char    fi1;
        char    fi2[2];
        char    fi3[2];
        char    fi4[4];
} dosdta;

typedef union {                 /* Overlaid by glob_t */
        dosdta  dt;
        glob_t  gt;
} dta_g;

typedef struct {                /* Plus other stuff we need */
        dta_g   g;
        int     nexti;          /* Index of next element in glob array */
        char    pathname[256];
} dta_t;

int     fgetfn_(dta_t **dta, char *filename)
{
        strcpy(filename, (*dta)->pathname);

        return (strlen(filename));
}

int     ffirst_(dta_t **dta, const char *fn, const int *attr)
{
        /* Returns: 0 for success, 1, 2 or 18 for failure */

        int     result;

        result = *attr;         // Just to reference it
        *dta = (dta_t *)malloc(sizeof (dta_t));
        if (*dta == 0)
                result = 1;
        else {
                result = glob(fn, GLOB_MARK, 0, &(*dta)->g.gt);
                if (result) {
                        free(*dta);
                        *dta = 0;
#ifdef ISF77
                        if (result == GLOB_ABEND)
#else
                        if (result == GLOB_NOMATCH)
#endif
                                result = 18;
                        else
                                result = 1;
                } else {
                        (*dta)->nexti = 1;
                        strcpy((*dta)->pathname, (*dta)->g.gt.gl_pathv[0]);
                }
        }

        return (result);
}
        
int     fnext_(dta_t **dta)
{
        /* Returns: 0 for success, 18 for no more matches */

        int     result;

        if (*dta && (*dta)->nexti < (*dta)->g.gt.gl_pathc) {
                result = 0;
                strcpy((*dta)->pathname,
                       (*dta)->g.gt.gl_pathv[(*dta)->nexti++]);
        } else {
                result = 18;
                if (*dta) {
                        globfree(&(*dta)->g.gt);
                        free(*dta);
                        *dta = 0;
                }
        }

        return (result);
}

int     rfisatty_(const int *handle)
{
        /* Returns: 1 if handle is that of a (character) device, else 0 */

#ifdef __MINGW32__
        return (isatty(*handle) > 0);
#else
        return (isatty(*handle));
#endif
}

int     mfretc_(const int *errcode)
{
        exit(*errcode);
}

#ifdef ISF77
int     rpargs_(char *args)
{
        /* Returns: string length, 0 if no args */

        int     argc, i, j, k;
        char    next_arg[128];

        *args = 0;
        argc = iargc_(0);
        j = 0;
        for (k = 1; k <= argc; ++k) {
                getarg_(&k, &next_arg[j], 128 - j);
                for (i = 127; i > 0 && isspace(next_arg[i]); --i)
                        ;
                next_arg[i + 1] = 0;
                strcat(args, next_arg);
                next_arg[0] = ' ';      /* 1 space between args */
                j = 1;
        }

        return (strlen(args));
}
#else
/* For f95. In FORTRAN, from is CHARACTER() and to is INTEGER*1().
 * This is just a hack to get around a type conversion in f95.
 * The iargc/getarg method above does not work in f95, so we
 * implement rpargs differently (in ratfor) and just use this
 * function to copy from a FORTRAN CHARACTER to INTEGER*1.
 * Return the number of characters transferred.
 */
int     rpxfer_(const char *from, char *to, const int *len)
{
        memcpy(to, from, *len);
        to[*len] = '\0';
        return (strlen(to));
}
#endif

int     rpclos_(const int *handle)
{
        return (close(*handle));
}

static void     ut2rt(const time_t *utime, int *rdate, int *rtime)
{
        struct tm       *tm_now;

        tm_now = localtime(utime);
        *rdate = 10000 * (1900 + tm_now->tm_year) +
                 100 * (1 + tm_now->tm_mon) + tm_now->tm_mday;
        *rtime = 10000 * tm_now->tm_hour +
                 100 * tm_now->tm_min + tm_now->tm_sec;
}

int     rpfdat_(const int *flag, const char *name, int *date, int *time)
{
        struct stat     sbuf;
        struct tm       tbuf;
        struct utimbuf  tb;
        int             hour, year;
        int             result;

        if (*flag == 0) {               /* Get */
                result = stat(name, &sbuf);
                if (result == 0) ut2rt(&sbuf.st_mtime, date, time);
        } else {                        /* Set */
                tbuf.tm_hour = *time / 10000;
                hour = 10000 * tbuf.tm_hour;
                tbuf.tm_min = (*time - hour) / 100;
                tbuf.tm_sec = *time - hour - 100 * tbuf.tm_min;
                year = *date / 10000;
                tbuf.tm_year = year - 1900;
                year *= 10000;
                tbuf.tm_mon = (*date - year) / 100 - 1;
                tbuf.tm_mday = *date - year - 100 * (1 + tbuf.tm_mon);
                tb.modtime = tb.actime = mktime(&tbuf);
                result = utime(name, &tb);
        }

        return (result);
}

int     rpgete_(const char *name, char *envstr)
{
        /* Returns: Length of string, 0 if not found */

        char    *p;
        int     result;

        p = getenv(name);
        if (p) {
                strcpy(envstr, p);
                result = strlen(p);
        } else {
                *envstr = 0;
                result = 0;
        }

        return (result);
}

int     rpnow_(int *the_date, int *the_time)
{
        /* Returns:     0 if no error, else error code
         *              date     as yyyymmdd (19980411)
         *              time     as hhmmss (24-hour clock)
         */

        time_t          now;
        int             result;

        now = time(0);
        if (now < 0)
                result = -1;
        else {
                result = 0;
                ut2rt(&now, the_date, the_time);
        }

        return (result);
}

int     rpremv_(const char *name)
{
        return (unlink(name));
}

#ifndef __MINGW32__
static int      isdir(const char *name)
{
        struct stat     sbuf;
        int             result = 0;

        if (stat(name, &sbuf) == 0 &&
            (sbuf.st_mode & S_IFMT) == S_IFDIR)
                result = 1;

        return (result);
}
#endif

int     rprenm_(const char *oldname, const char *newname)
{
#ifdef __MINGW32__
        return rename(oldname, newname);
#else
        if (!isdir(newname))
                if (unlink(newname) == -1 && errno != ENOENT)
                        return (-1);
        if (link(oldname, newname) == -1)
                return (-1);
        if (unlink(oldname) == -1)
                return (-1);

        return (0);
#endif
}

int     rpopen_(const char *name, const int *mode)
{
        int     flags;

        switch (*mode & 3) {
          case 0:       flags = O_RDONLY; break;
          case 1:       flags = O_WRONLY; break;
          case 2:       flags = O_RDWR; break;
          default:      flags = O_RDONLY;
        }
        if (*mode & 256) flags |= (O_CREAT | O_TRUNC);

#ifdef __MINGW32__
        if (*mode & 4) {       /* BINMODE */
                flags &= ~_O_TEXT;
                flags |= _O_BINARY;
        }
#endif

        return (open(name, flags, 0666));
}

int     rpread_(const int *handle, char *buffer, const int *count)
{
        return (read(*handle, buffer, (size_t)*count));
}

long    rpseek_(const int *handle, const long *offset, const int *origin)
{
#ifdef __APPLE__
        return ((long)lseek(*handle, (off_t)*offset, *origin));
#else
#ifdef __MINGW32__
        return ((long)lseek(*handle, (off_t)*offset, *origin));
#else
        return ((long)lseek(*handle, (__off_t)*offset, *origin));
#endif
#endif
}

int     rpwrit_(const int *handle, const char *buffer, const int *count)
{
        return (write(*handle, buffer, (size_t)*count));
}

char    int2ch_(const int *i)
{
        return (char)(*i & 255);
}
